from typing import List, Union, Literal

from collinear.BaseService import BaseService
from collinear.judge.types import VeritasNanoOutput, ConversationMessage, ConversationInput, VeritasOutput


class Veritas(BaseService):
    def __init__(self, access_token: str) -> None:
        super().__init__(access_token)

    async def question_answer(self, context: str, question: str, answer: str,
                              model_name: Literal[
                                  'veritas_nano', 'veritas'] = 'veritas_nano') -> VeritasNanoOutput | VeritasOutput:
        """
       Sends a question-answer request to the veritas judge.

       Args:
        context (str): The context of the question.
        question (str): The question to answer.
        answer (str): The answer provided.
        model_name (Literal['veritas_nano', 'veritas']): The model to use for the judgement. Default is 'veritas_nano'.

       Returns:
           VeritasNanoOutput: Contains judgement and score.

       """
        response = await self.send_request('/api/v1/judge/reliability', "POST", {
            'model_name': model_name,
            'type': 'qa',
            "context": context,
            "question": question,
            "answer": answer
        })
        if model_name == 'veritas_nano':
            return VeritasNanoOutput(judgement=response['judgement'], score=response['extra']['score'])
        else:
            return VeritasOutput(judgement=response['judgement'], rationale=response['extra']['rationale'])

    async def natural_language_inference(self, context: str, claim: str, model_name: Literal[
        'veritas_nano', 'veritas'] = 'veritas_nano') -> VeritasNanoOutput | VeritasOutput:
        """
        Sends a natural language inference request to the veritas judge.

        Args:
            context (str): The context of the claim.
            claim (str): The claim to check.
            model_name (Literal['veritas_nano', 'veritas']): The model to use for the judgement. Default is 'veritas_nano'.


        Returns:
            VeritasNanoOutput: Contains judgement and score.

        """
        response = await self.send_request('/api/v1/judge/reliability', "POST", {
            'model_name': model_name,
            'type': 'nli',
            "context": context,
            "claim": claim
        })
        if model_name == 'veritas_nano':
            return VeritasNanoOutput(judgement=response['judgement'], score=response['extra']['score'])
        else:
            return VeritasOutput(judgement=response['judgement'], rationale=response['extra']['rationale'])

    async def conversation(self, context: str, conv_prefix: List[Union[ConversationMessage, dict]],
                           response: Union[ConversationMessage, dict],
                           model_name: Literal[
                               'veritas_nano', 'veritas'] = 'veritas_nano') -> VeritasNanoOutput | VeritasOutput:
        """
    Sends a conversation request to the Veritas judge.

    Args:
        context (str): The context of the conversation. Provide relevant background information.
        conv_prefix (List[Union[ConversationMessage, dict]]): The conversation history.
            Each message should be either a ConversationMessage object or a dict with 'role' and 'content' keys.
            Example: [{"role": "user", "content": "Hello"}, {"role": "assistant", "content": "Hi there!"}]
        response (Union[ConversationMessage, dict]): The response to be evaluated.
            Should be either a ConversationMessage object or a dict with 'role' and 'content' keys.
            Example: {"role": "assistant", "content": "How can I help you today?"}
        model_name (Literal['veritas_nano', 'veritas']): The model to use for the judgement. Default is 'veritas_nano'.


    Returns:
        VeritasNanoOutput: Contains the judgement and score from the Veritas judge.

    Raises:
        ValueError: If the input data is invalid or doesn't meet the required format.

    Example:
        output = await conversation(
            context="Customer service conversation",
            conv_prefix=[{"role": "user", "content": "I have a problem with my order"}],
            response={"role": "assistant", "content": "I'm sorry to hear that. Can you provide more details?"}
        )
    """
        # NOTE: Doing this to validate input data
        input_data = ConversationInput(
            context=context,
            conversation=conv_prefix,
            response=response
        )

        response = await self.send_request('/api/v1/judge/reliability', "POST", {
            'model_name': model_name,
            'type': 'conversation',
            "context": context,

            **input_data.dict()
        })
        if model_name == 'veritas_nano':
            return VeritasNanoOutput(judgement=response['judgement'], score=response['extra']['score'])
        else:
            return VeritasOutput(judgement=response['judgement'], rationale=response['extra']['rationale'])



import uuid

import pandas as pd

from collinear.BaseService import BaseService


class Dataset(BaseService):
    def __init__(self, access_token: str) -> None:
        super().__init__(access_token)

    async def upload_dataset(self, data: pd.DataFrame,
                             conv_prefix_column_name: str,
                             response_column_name: str,
                             judgement_column_name: str | None,
                             dataset_name: str,
                             space_id: uuid.UUID,
                             parent_dataset_id: uuid.UUID | None) -> uuid.UUID:
        req_obj = {
            "name": dataset_name,
            "space_id": space_id,
        }
        if parent_dataset_id:
            req_obj['parent_dataset_id'] = parent_dataset_id
        conversations = []
        for index, row in data.iterrows():
            obj = {
                'conv_prefix': list(row[conv_prefix_column_name]),
                'response': row[response_column_name]['content'],
                'judgements': row[judgement_column_name] if judgement_column_name in row else {}
            }
            conversations.append(obj)
        req_obj['conversations'] = conversations
        output = await self.send_request('/api/v1/dataset', "POST", req_obj)
        print(output)
        return output['dataset_id']



from typing import Literal

import aiohttp


class BaseService:
    def __init__(self, access_token: str) -> None:
        self.base_url = 'https://api.collinear.ai'
        self.access_token = access_token

    def set_access_token(self, access_token: str):
        """
        Sets the access token for the entire SDK.
        """
        self.access_token = access_token
        return self

    def get_headers(self) -> dict:
        return {
            "Authorization": f"Bearer {self.access_token}",
            "Content-Type": "application/json"
        }

    async def send_request(self, url: str, method: Literal["POST", "GET", "PUT", "DELETE", "PATCH"] = "GET",
                           data: dict = None) -> dict:
        full_url = f"{self.base_url}{url}"
        async with aiohttp.ClientSession() as session:
            async with session.request(
                    method=method,
                    url=full_url,
                    headers=self.get_headers(),
                    json=data
            ) as response:
                try:
                    response_data = await response.json()
                    return response_data
                except Exception as e:
                    print(f"Failed to parse JSON response: {e}")
                    return {}



import uuid
from enum import Enum

from pydantic import BaseModel


class ModelTypeEnum(Enum):
    openai = "openai"
    anthropic = "anthropic"


class ModelDTO(BaseModel):
    id: uuid.UUID
    name: str
    nickname: str
    base_url: str
    api_key: str
    type: ModelTypeEnum



from typing import Literal, List, Optional

from pydantic import BaseModel


class ConversationMessage(BaseModel):
    """
       Represents a single message in a conversation.

       Attributes:
           content (str): The text content of the message.
           role (Literal["user", "assistant", "system"]): The role of the entity sending the message.
               Must be one of "user", "assistant", or "system".

       Example:
           >>> message = ConversationMessage(content="Hello, how can I help?", role="assistant")
       """
    content: str
    role: Literal["user", "assistant", "system"]


class ConversationInput(BaseModel):
    context: str
    conversation: List[ConversationMessage]
    response: ConversationMessage

    class Config:
        populate_by_name = True


class VeritasNanoOutput(BaseModel):
    """
       Represents the output from the Veritas judgement system.

       Attributes:
           judgement (int): The numerical judgment. 1 means the input is factual, 0 means it is hallucinated.
           score (float): A float value representing an additional scoring metric.

       Example:
           >>> output = VeritasNanoOutput(judgement=8, score=0.95)
       """
    judgement: int
    score: float


class VeritasOutput(BaseModel):
    """
       Represents the output from the Veritas Judge.

       Attributes:
           judgement (int): The numerical judgement. 1 means the input is factual, 0 means it is hallucinated.
           rationale (str): The rationale for the judgement.

       Example:
           >>> output = VeritasNanoOutput(judgement=8, score=0.95)
       """
    judgement: int
    rationale: str


class CollinearGuardNanoOutput(BaseModel):
    """
    Represents the output from the Collinear Guard Nano system.

    Attributes:
        judgement (int): The numerical judgement. 1 means the input is safe, 0 means it is unsafe.

    Example:
        >>> output = CollinearGuardNanoOutput(judgement=8)
    """
    judgement: int


class ScoringCriteria(BaseModel):
    """
    Represents the scoring criteria for the Collinear Guard.

    Attributes:
        score (float): The score to assign to a scoring criteria
        description (str): Description you want to give to the scoring criteria

    Example:
        >>> criteria = ScoringCriteria(factual_score=1.0, hallucinated_score=0.0)
    """
    score: int
    description: str


class CollinearGuardPointwiseOutput(BaseModel):
    """
    Represents the output from the Collinear Guard model.

    Attributes:
        judgement (int): The numerical judgement. It can range based on scoring criteria.
        rationale (str): The rationale for the judgement.
    Examples:
          >>> output = CollinearGuardPointwiseOutput(judgement=5, rationale="The response is safe.")

    """
    judgement: int
    rationale: str


class ClassificationInput(BaseModel):
    nano_model_type: Literal['prompt', 'response', 'refusal']
    conversation: List[ConversationMessage]
    response: ConversationMessage
    scoring_criteria: Optional[List[ScoringCriteria]] = None


class PointwiseInput(BaseModel):
    conversation: List[ConversationMessage]
    response: ConversationMessage
    scoring_criteria: Optional[List[ScoringCriteria]] = None
